package com.zh.project0512.controller.manage;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.mapper.AdminMapper;
import com.zh.project0512.model.Video;
import com.zh.project0512.model.dto.RankListDTO;
import com.zh.project0512.service.IRankService;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;

/**
 * 首页数据统计
 */
@Tag(name = "首页数据统计")
@RestController
@RequestMapping("/manage/statistics")
public class StatisticsController {
    @Autowired
    private AdminMapper adminMapper;
    @Resource
    IRankService rankService;

    @Operation(summary = "总数据")
    @PostMapping("/data1")
    @AdminTokenValid
    public Result data1() {
        return Result.success(adminMapper.statistics1(), "请求成功");
    }

    @Data
    static class SSParam {
        @Schema(title = "时间条件", description = "1今日;2本周;3本月;4全年")
        private Integer dateCondition;
        @Schema(title = "开始时间(yyyy-MM-dd HH:mm:ss)", description = "如果传入dateCondition以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime;
        @Schema(title = "结束时间(yyyy-MM-dd HH:mm:ss)", description = "如果传入dateCondition以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime;
    }

    @Operation(summary = "视频数据")
    @PostMapping("/data2")
    @AdminTokenValid
    public Result data2(@Validated @RequestBody SSParam param) {

        QueryWrapper<Video> qw = new QueryWrapper<>();
        if (null != param.getDateCondition()) {
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime before = null;
            switch (param.getDateCondition()) {
                case 1:
                    before = now;
                    break;
                case 2:
                    before = now.minusWeeks(1);
                    break;
                case 3:
                    before = now.minusMonths(1);
                    break;
                case 4:
                    before = now.minusYears(1);
                    break;
                default:
            }
            qw.between("creatAt", before, LocalDateTime.now());
        } else {
            qw.ge(param.getBeginTime() != null, "date_format(creatAt, '%Y-%m-%d')", param.getBeginTime());
            qw.le(param.getEndTime() != null, "date_format(creatAt, '%Y-%m-%d')", param.getEndTime());
        }
        List<JSONObject> videoDataBydDay = adminMapper.statistics2(qw);
        ArrayList<JSONObject> res = new ArrayList<>();
        JSONObject first = videoDataBydDay.get(0);
        JSONObject last = videoDataBydDay.get(videoDataBydDay.size() - 1);
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDate dat = LocalDate.parse((CharSequence) first.get("dat"), df);
        LocalDate dat2 = LocalDate.parse((CharSequence) last.get("dat"), df);
        for (int i = 0; i <= ChronoUnit.DAYS.between(dat, dat2); i++) {
            LocalDate today = dat.plusDays(i);
            JSONObject obj = null;
//            Stream<JSONObject> dat1 = videoDataBydDay.stream().filter(x -> LocalDate.parse((CharSequence) x.get("dat"), df) == today);
            for (JSONObject o : videoDataBydDay) {
                if (LocalDate.parse((CharSequence) o.get("dat"), df).equals(today)) {
                    obj = o;
                    break;
                }
            }
            res.add(obj == null ? new JSONObject().fluentPut("dat", today) : obj);
        }
        return Result.success(res, "请求成功");
    }

    @Operation(summary = "排行列表")
    @PostMapping("/data3")
    @AdminTokenValid
    public Result rankList(@Validated @RequestBody RankListDTO rankListDTO) {
        return Result.success(rankService.rankList(rankListDTO), "请求成功");
    }
}
