package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.TopicActivity;
import com.zh.project0512.service.ITopicActivityService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@Tag(name = "动态")
@RestController
@RequestMapping("/manage/topicActivity")
public class TopicActivityController {
    @Autowired
    private ITopicActivityService topicActivityService;

    @Data
    static class AddTopicParam {
        @Schema(title = "标题")
        @NotNull(message = "title不能为空")
        private String title;
        @Schema(title = "副标题")
        private String subtitle;
        @Schema(title = "类型", description = "1视频；2图片；3文章")
        @NotNull(message = "type不能为空")
        private Integer type;
        @NotNull(message = "content不能为空")
        private String content;
        @Schema(title = "时长", description = "视频文件请上传时长")
        private Integer duration;
        @Schema(title = "封面")
        private String cover;
        @Schema(title = "是否首页显示",description = "0否；1是")
        private Integer showAtIndex;
    }

    @Operation(summary = "新增动态")
    @PostMapping("/add")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "4")
    public Result add(@Validated @RequestBody AddTopicParam param) {
        TopicActivity topicActivity = new TopicActivity().setTitle(param.getTitle()).setSubtitle(param.getSubtitle())
            .setType(param.getType()).setContent(param.getContent()).setDuration(param.getDuration()).setCover(param.getCover()).setCreatAt(LocalDateTime.now());
        if(param.getShowAtIndex() !=null && param.getShowAtIndex() == 1){
            topicActivity.setShowAtIndex(1);
        }
        topicActivityService.save(topicActivity);
        return Result.success("添加完成");
    }

    @Data
    static class DelTopicParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
    }

    @Operation(summary = "删除动态")
    @PostMapping("/del")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "5")
    public Result del(@Validated @RequestBody DelTopicParam param) {
        return MybatisPlusUtil.sqlResult(topicActivityService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdTopicParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
        @Schema(title = "标题")
        private String title;
        @Schema(title = "内容")
        private String content;
        @Schema(title = "类型", description = "1视频；2图片；3文章")
        private Integer type;
        @Schema(title = "封面")
        private String cover;
    }

    @Operation(summary = "更新动态内容")
    @PostMapping("/upd")
    @AdminTokenValid
    public Result upd(@Validated @RequestBody UpdTopicParam param) {
        return MybatisPlusUtil.sqlResult(topicActivityService.updateById(new TopicActivity().setId(param.getId()).setTitle(param.getTitle()).setType(param.getType()).setCover(param.getCover()).setContent(param.getContent()).setUpdateAt(LocalDateTime.now())), "修改");
    }

    @Operation(summary = "动态列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@RequestBody(required = false) JSONObject obj) {
        QueryWrapper<TopicActivity> qw = new QueryWrapper<>();
        qw.orderByDesc("updateAt","creatAt");
        if(obj.getBoolean("showAtIndex") !=null && obj.getBoolean("showAtIndex") == true){
            qw.eq("showAtIndex",1);
        }
        return Result.success(topicActivityService.pageMaps(MybatisPlusUtil.SetPage(obj),qw));
    }

    @Data
    static class DetTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("/detail")
    @AdminTokenValid
    public Result detail(@Validated @RequestBody TaskController.DetTaskParam param) {
        int id =param.getId();
        TopicActivity topicActivity = topicActivityService.getById(id);
        return topicActivity != null ? Result.success(topicActivity) : Result.fail(HttpStatusEnum.NOT_FOUND);
    }

    @Data
    static class showAtIndexParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
        @NotNull(message = "showAtIndex不能为空")
        @Min(value = 0, message = "取值是：0否；1是")
        @Max(value = 1, message = "取值是：0否；1是")
        @Schema(title = "是否首页显示",description = "0否；1是")
        private Integer showAtIndex;
    }

    @Operation(summary = "修改动态是否首页显示")
    @PostMapping("/showAtIndex")
    @AdminTokenValid
    public Result showAtIndex(@Validated @RequestBody showAtIndexParam param) {
        UpdateWrapper<TopicActivity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", param.getId()).set("showAtIndex",param.getShowAtIndex());
        return MybatisPlusUtil.sqlResult(topicActivityService.update(updateWrapper),"编辑");
    }

}
