package com.zh.project0512.controller.wxApp;

import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.model.dto.RankListDTO;
import com.zh.project0512.service.IRankService;
import com.zh.project0512.service.IVideoService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * 排行榜
 */
@Tag(name = "排行榜")
@RestController
@RequestMapping("/wxApp/rank")
public class RankUController {
    @Resource
    IRankService rankService;
    @Autowired
    private IVideoService videoService;

    /**
     * 查询排行榜列表
     *
     * @param rankListDTO 排行列表DTO
     * @return 排行榜列表
     */
    @Operation(summary = "排行列表")
    @PostMapping("/list")
    public Result rankList(@Validated @RequestBody RankListDTO rankListDTO) {
        return Result.success(rankService.rankList(rankListDTO), "请求成功");
    }

    @Operation(summary = "个人总记录")
    @PostMapping("/myEffect")
    @TokenValid
    public Result myEffect(@RequestHeader String token) {
        return Result.success(videoService.record(new JwtUtil().parseOpenid(token)), "请求成功");
    }
}
