package com.zh.project0512.controller.wxApp;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.model.*;
import com.zh.project0512.model.vo.CommunicationEffectVo;
import com.zh.project0512.service.*;
import com.zh.project0512.utils.CustomUtil;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-21
 */
@Tag(name = "任务管理")
@Validated
@RestController
@RequestMapping("/wxApp/task")
public class TaskUController {
    @Autowired
    private IUserService userService;
    @Autowired
    private ITaskService taskService;
    @Autowired
    private IUserTaskService userTaskService;
    @Autowired
    private ITaskTagService taskTagService;
    @Autowired
    private ITaskBrandService taskBrandService;
    @Autowired
    private ITaskReferenceService taskReferenceService;
    @Autowired
    private IRewardRuleService rewardRuleService;
    @Autowired
    private IRewardRuleTemplateService rewardRuleTemplateService;
    @Autowired
    private ICustomerActionNoteService customerActionNoteService;

    @Data
    static class ListTParam {
        private int pageNum;
        private int pageSize;
        @Schema(title = "用户的任务状态", description = "0待领取；1进行中；2已完成")
        @Min(value = 0)
        @Max(value = 2)
        private Integer status;
        @Schema(title = "月份区间")
        private Integer month;
        @Schema(title = "开始时间(yyyy-MM-dd HH:mm:ss)",description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime;
        @Schema(title = "结束时间(yyyy-MM-dd HH:mm:ss)",description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime;
        @Schema(title = "tagId数组")
        private List<Integer> tagIdList;
        @Schema(title = "brandId数组")
        private List<Integer> brandIdList;
        @Schema(title = "任务名称关键词搜索")
        private String keyword;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @TokenValid
    public Result list(@Validated @RequestBody ListTParam param, @RequestHeader String token) {
        QueryWrapper<Reference> qw = new QueryWrapper<>();
        Integer status = param.getStatus();
        String openid = new JwtUtil().parseOpenid(token);
        qw.ne("t1.status",0);
        if (null != status) {
            qw.eq("t1.status", status == 2 ? 2 : 1);
            if(status == 0){
                qw.ne("(SELECT count(*)!=0 from userTask as t4,user as t5 WHERE t1.id = t4.taskId and t4.userId = t5.id and t5.openid = +'"+openid+"' )", 1);
            }
           else if(status == 1){
                qw.eq("(SELECT count(*)!=0 from userTask as t4,user as t5 WHERE t1.id = t4.taskId and t4.userId = t5.id and t5.openid = +'"+openid+"' )", 1);
            }
        }
        if (null != param.getMonth()) {
            qw.between("t1.creatAt", LocalDateTime.now().minusMonths(param.getMonth()), LocalDateTime.now());
        }else{
            qw.ge(param.getBeginTime()!=null,"t1.creatAt",param.getBeginTime());
            qw.le(param.getEndTime()!=null,"t1.creatAt",param.getEndTime());
        }
        if (null != param.getKeyword()) {
            qw.like("t1.title", param.getKeyword());
        }
        IPage iPage = taskService.idList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), openid,param.getTagIdList(), param.getBrandIdList(), qw);
        List list = iPage.getRecords();
        if (list.size() > 0) {
            iPage.setRecords(taskService.list(new JwtUtil().parseOpenid(token), list, param.getTagIdList(), param.getBrandIdList(), qw));
        }
        return Result.success(iPage);
    }

    @Data
    static class DetTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("detail")
    public Result det(@Validated @RequestBody DetTaskParam param, @RequestHeader(required = false, value = "token") @Parameter(name = "用户token", description = "关联用户信息") String token) {
        Task task = taskService.detailById(new JwtUtil().parseOpenid(token), param.getId());
        JSONObject res = (JSONObject) JSONObject.toJSON(task);
        if(task!=null) {
            List rewardRulesList = rewardRuleService.listByTemplateId(task.getRewardRuleTemplateId());
            res.put("rewardRuleList", rewardRulesList);
        }
        return Result.success(res);
    }

    @Data
    static class CusNoteParam {
        @Schema(title = "任务id")
        private Integer taskId;
        @NotNull(message = "顾客姓名不能为空")
        @Schema(title = "顾客姓名")
        private String customerName;
        @NotNull(message = "联系方式不能为空")
        private String tel;
        @Schema(title = "地区")
        @NotNull(message = "地区不能为空")
        private String area;
        @Schema(title = "车型列表")
        private List<String> brandList;
        @Schema(title = "备注")
        private String remarks;
    }

    @Operation(summary = "用户线索搜集")
    @PostMapping("/customerNote")
    @TokenValid
    public Result customerNote(@Validated @RequestBody CusNoteParam param, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Integer useId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (useId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        if(param.getTaskId()!=null) {
            Integer taskStatus = userTaskService.userTaskStatus(useId, param.getTaskId());
            if (taskStatus == null) {
                return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "用户未接受任务");
            } else if (taskStatus == 0) {
                return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "任务未开始");
            } else if (taskStatus == 2) {
                return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "任务已结束");
            }
        }
        if(null !=customerActionNoteService.getOne(new QueryWrapper<CustomerActionNote>().eq("tel",param.getTel()))){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "该手机用户已在线索里");
        }
        customerActionNoteService.save(
            new CustomerActionNote().setCustomerName(param.getCustomerName()).setTel(param.getTel()).setArea(param.getArea()).setTaskId(param.getTaskId())
                .setBrandList(new CustomUtil().listToString(param.getBrandList())).setRemarks(param.getRemarks()).setProvideId(useId).setCreatAt(LocalDateTime.now()));
        return Result.success("上传完成");
    }

    @Operation(summary = "线索搜集列表")
    @PostMapping("/customerNoteList")
    @TokenValid
    public Result customerNote(@Validated @RequestBody JSONObject obj, @RequestHeader(value = "token") @Parameter(name = "登录token") String token) {
        QueryWrapper<CustomerActionNote> qw = new QueryWrapper<>();
        String keywords = obj.getString("keyword");
        if(keywords != null) {
            qw.and(
                wrapper ->
                    wrapper.like("t1.customerName", keywords).or().like("t3.title", keywords).like("t1.tel", keywords)
                        .or().like("t1.area", keywords).or().like("t1.remarks", keywords).or().like("t1.brandList", keywords)
            );
        }
        IPage<Map> iPage = customerActionNoteService.pageList(MybatisPlusUtil.SetPage(obj), new JwtUtil().parseOpenid(token),qw);
        List<Map> records = iPage.getRecords();
        for (Map m : records) {
            if (m.get("brandList") != null) {
                m.replace("brandList", new CustomUtil().stringToList(m.get("brandList").toString()));
            }
        }
        return Result.success(iPage);
    }

    /**
     * 传播效果
     * @param token token
     * @return 传播效果
     */
    @Operation(summary = "传播效果")
    @PostMapping("/communicationEffect")
    @TokenValid
    public Result<CommunicationEffectVo> communicationEffect(@RequestHeader(value = "token") String token) {
        String openid = new JwtUtil().parseOpenid(token);
        return Result.success(customerActionNoteService.communicationEffect(openid),"请求成功");
    }
}

