package com.zh.project0512.serviceImpl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.project0512.mapper.*;
import com.zh.project0512.model.*;
import com.zh.project0512.model.vo.CommunicationEffectVo;
import com.zh.project0512.service.ICustomerActionNoteService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zh
 * @since 2022-05-30
 */
@Service
public class CustomerActionNoteServiceImpl extends ServiceImpl<CustomerActionNoteMapper, CustomerActionNote> implements ICustomerActionNoteService {
    @Resource
    CustomerActionNoteMapper customerActionNoteMapper;
    @Resource
    UserMapper userMapper;
    @Resource
    VideoMapper videoMapper;
    @Resource
    RewardRuleTemplateMapper rewardRuleTemplateMapper;
    @Resource
    RewardRuleMapper rewardRuleMapper;

   public void add(CustomerActionNote customerActionNote){
        customerActionNoteMapper.add(customerActionNote);
    }
    public IPage<Map> pageList(IPage page,String openid, @Param("ew") Wrapper<CustomerActionNote> queryWrapper){
        return customerActionNoteMapper.pageList(page,openid,queryWrapper);
    }

    @Override
    public CommunicationEffectVo communicationEffect(String openid) {
        CommunicationEffectVo communicationEffectVo = new CommunicationEffectVo();

        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("openid",openid);
        User user = userMapper.selectOne(queryWrapper);

        //平台：1抖音；2快手；3朋友圈；4视频号
        Video dyVideo = findVideoByUserId(user.getId(),1);
        communicationEffectVo.setDyPlayNum(dyVideo.getPlayNum());
        communicationEffectVo.setDyCommendNum(dyVideo.getCommendNum());
        communicationEffectVo.setDyCollectionNum(dyVideo.getCollectionNum());
        communicationEffectVo.setDyCommentNum(dyVideo.getCommentNum());
        Video ksVideo = findVideoByUserId(user.getId(),2);
        communicationEffectVo.setKsPlayNum(ksVideo.getPlayNum());
        communicationEffectVo.setKsCommendNum(ksVideo.getCommendNum());
        communicationEffectVo.setKsCollectionNum(ksVideo.getCollectionNum());
        communicationEffectVo.setKsCommentNum(ksVideo.getCommentNum());
        Video pyqVideo = findVideoByUserId(user.getId(),3);
        communicationEffectVo.setPyqCommendNum(pyqVideo.getCommendNum());
        communicationEffectVo.setPyqCommentNum(pyqVideo.getCommentNum());
        Video sphVideo = findVideoByUserId(user.getId(),4);
        communicationEffectVo.setSphCommendNum(sphVideo.getCommendNum());
        communicationEffectVo.setSphReSendNum(sphVideo.getReSendNum());
        communicationEffectVo.setSphRecommendNum(sphVideo.getRecommendNum());
        communicationEffectVo.setSphCommentNum(sphVideo.getCommentNum());

        //查询默认模版
        QueryWrapper<RewardRuleTemplate> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.eq("isDefault",1);
        RewardRuleTemplate rewardRuleTemplate = rewardRuleTemplateMapper.selectOne(queryWrapper3);
        if (rewardRuleTemplate == null){
//            Integer communicationNumAll = communicationEffectVo.getCommunicationNumAll();
            communicationEffectVo.setPoints(0);
            return communicationEffectVo;
        }

        Integer points = findPointByCommunicationNum(rewardRuleTemplate.getId(),communicationEffectVo.getCommunicationNumAll());
        communicationEffectVo.setPoints(points);
        return communicationEffectVo;
    }

    /**
     * 根据传播值查询积分
     * @param templateId 默认模版主键Id
     * @param communicationNum 传播值
     * @return 积分
     */
    private Integer findPointByCommunicationNum(Integer templateId, Integer communicationNum) {
        QueryWrapper<RewardRule> queryWrapper4 = new QueryWrapper<>();
        queryWrapper4.eq("templateId",templateId);
        queryWrapper4.le("limitNum",communicationNum);
        queryWrapper4.orderByDesc("limitNum");
        queryWrapper4.last("limit 1");
        RewardRule rewardRule = rewardRuleMapper.selectOne(queryWrapper4);
        if (rewardRule == null){
            return 0;
        }
        return rewardRule.getReward();
    }

    /**
     * 根据查询视频信息
     * @param userId 用户主键id
     * @param type 平台：1抖音；2快手；3朋友圈；4视频号
     * @return 视频信息
     */
    private Video findVideoByUserId(Integer userId, int type) {
        QueryWrapper<Video> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("type",type);
        queryWrapper2.eq("userId",userId);
        queryWrapper2.select("IFNULL(SUM(playNum),0) as playNum,IFNULL(SUM(commendNum),0) as commendNum,IFNULL(SUM(collectionNum),0) as collectionNum,IFNULL(SUM(commentNum),0) as commentNum,IFNULL(SUM(reSendNum),0) as reSendNum,IFNULL(SUM(recommendNum),0) as recommendNum");
        return videoMapper.selectOne(queryWrapper2);
    }
}
